@@ -19,7 +19,7 @@
 #' @references Rodriguez-Rondon, Gabriel and Jean-Marie Dufour. 2023. “Monte Carlo Likelihood Ratio Tests for Markov Switching Models.” \emph{Unpublished manuscript}.
 #'
 #' @export
-LR_samp_dist_par <- function(mdl_h0, k1, N, burnin, mdl_h0_control, mdl_h1_control, workers){ 
+LR_samp_dist_par <- function(mdl_h0, k1, N, burnin, Z, mdl_h0_control, mdl_h1_control, workers){ 
   # ----- Set number of simulations per worker
   N_i_f <- floor(N/workers)
   N_worker_i <- matrix(rep(N_i_f, workers), workers, 1)
@@ -31,7 +31,7 @@ LR_samp_dist_par <- function(mdl_h0, k1, N, burnin, mdl_h0_control, mdl_h1_contr
   `%dopar%` <- foreach::`%dopar%`
   wi <- NULL # needed to pass CMD check
   LRN_all <- foreach::foreach(wi = 1:workers, .inorder = FALSE, .packages = "MSTest") %dopar% {
-    LRN <- LR_samp_dist(mdl_h0, k1, N_worker_i[wi], burnin, mdl_h0_control, mdl_h1_control) 
+    LRN <- LR_samp_dist(mdl_h0, k1, N_worker_i[wi], burnin, Z, mdl_h0_control, mdl_h1_control) 
     LRN
   }
   return(unlist(LRN_all))
@@ -48,6 +48,7 @@ LR_samp_dist_par <- function(mdl_h0, k1, N, burnin, mdl_h0_control, mdl_h1_contr
 #' @param p integer specifying the number of autoregressive lags.
 #' @param q integer specifying the number of series.
 #' @param k integer specifying the number of regimes.
+#' @param Z exogeneous regressors. Defualt is NULL.
 #' @param control List with control options for model estimation. For default values, see description of model being estimated.
 #' 
 #' @return List with estimated model properties. 
@@ -55,34 +56,54 @@ LR_samp_dist_par <- function(mdl_h0, k1, N, burnin, mdl_h0_control, mdl_h1_contr
 #' @keywords internal
 #' 
 #' @export
-estimMdl <- function(Y, p, q, k, control = list()){
+estimMdl <- function(Y, p, q, k, Z = NULL, control = list()){
   if ((k==1) & (p==0)){
     # Normally distributed model
     control$const <- TRUE # forced to be TRUE for hypothesis testing
-    mdl <- Nmdl(Y, control)
+    mdl <- Nmdl(Y, Z, control)
     mdl$converged = TRUE
   }else if ((k>1) & (p==0)){
     # Hidden Markov model
-    mdl <- HMmdl(Y, k, control)
+    mdl <- HMmdl(Y, k, Z, control)
     mdl$converged = (mdl$deltath <= mdl$control$thtol)
   }else if ((k==1) & (q==1) & (p>0)){
     # Autoregressive model
     control$const <- TRUE # forced to be TRUE for hypothesis testing
-    mdl <- ARmdl(Y, p, control)
-    mdl$converged = TRUE
+    if (is.null(Z) | (length(Z)==0)){
+      mdl <- ARmdl(Y, p, control)
+      mdl$converged = TRUE 
+    }else{
+      mdl <- ARXmdl(Y, p, Z, control)
+      mdl$converged = TRUE
+    }
   }else if ((k>1) & (q==1) & (p>0)){
     # Markov switching model
-    mdl <- MSARmdl(Y, p, k, control)
-    mdl$converged = (mdl$deltath <= mdl$control$thtol)
+    if (is.null(Z) | (length(Z)==0)){
+      mdl <- MSARmdl(Y, p, k, control)
+      mdl$converged = (mdl$deltath <= mdl$control$thtol)  
+    }else{
+      mdl <- MSARXmdl(Y, p, k, Z, control)
+      mdl$converged = (mdl$deltath <= mdl$control$thtol)
+    }
   }else if ((k==1) & (q>1) & (p>0)){
     # Vector autoregressive model
     control$const <- TRUE # forced to be TRUE for hypothesis testing
-    mdl <- VARmdl(Y, p, control)
-    mdl$converged = TRUE
+    if (is.null(Z) | (length(Z)==0)){
+      mdl <- VARmdl(Y, p, control)
+      mdl$converged = TRUE 
+    }else{
+      mdl <- VARXmdl(Y, p, Z, control)
+      mdl$converged = TRUE 
+    }
   }else if ((k>1) & (q>1) & (p>0)){
     # Vector autoregressive Markov switching model
-    mdl <- MSVARmdl(Y, p, k, control)
-    mdl$converged = (mdl$deltath <= mdl$control$thtol)
+    if (is.null(Z) | (length(Z)==0)){
+      mdl <- MSVARmdl(Y, p, k, control)
+      mdl$converged = (mdl$deltath <= mdl$control$thtol)
+    }else{
+      mdl <- MSVARXmdl(Y, p, k, Z, control)
+      mdl$converged = (mdl$deltath <= mdl$control$thtol)
+    }
   }
   return(mdl)
 }
@@ -91,17 +112,20 @@ estimMdl <- function(Y, p, q, k, control = list()){
 #' @title Monte Carlo Likelihood Ratio Test
 #' 
 #' @description This function performs the Local Monte Carlo likelihood ratio 
-#' test (LMC-LRT) proposed in Rodriguez-Rondon & Dufour (2022).
+#' test (LMC-LRT) proposed in Rodriguez-Rondon & Dufour (2024). As discussed in 
+#' their work, this test can be applied in very general settings and can be used 
+#' to compare varioous regimes under the null and under the alternative. 
 #' 
-#' @param Y  Series to be tested. Must be a (\code{T x q}) matrix.
+#' @param Y  Series to be tested. Must be a (\code{T x q}) matrix where T is the number of time observations and q is the number of variables.
 #' @param p  Number of autoregressive lags. Must be greater than or equal to 0. 
 #' @param k0 Number of regimes under null hypothesis. Must be greater than or equal to 1.
 #' @param k1 Number of regimes under alternative hypothesis. Must be greater than \code{k0}.
+#' @param Z Exogenous regressors. Optional input and default is NULL. When used, it should be a (\code{T x qz}) matrix where T is the number of time observations and q is the number of exogenous variables.
 #' @param control List with test procedure options including: 
 #' \itemize{
 #'   \item N: Integer determining the number of Monte Carlo simulations. Default is set to \code{99} as in paper.
 #'   \item burnin: Number of simulated observations to remove from beginning. Default is \code{100}.
-#'   \item converge_check: String of NULL determining if convergence of model(s) should be verified. Allowed inputs are: "null", "alt", "both", or \code{NULL}. If \code{NULL} (default) no model convergence is verified.
+#'   \item converge_check: String or NULL determining if convergence of model(s) should be verified. Allowed inputs are: "null", "alt", "both", or \code{NULL}. If \code{NULL} (default) no model convergence is verified.
 #'   \item workers: Integer determining the number of workers to use for parallel computing version of test. Note that parallel pool must already be open. Default is \code{0}.
 #'   \item mdl_h0_control: List with restricted model options. See \code{\link{Nmdl}}, \code{\link{ARmdl}}, \code{\link{VARmdl}}, \code{\link{HMmdl}}, \code{\link{MSARmdl}}, or \code{\link{MSVARmdl}} documentation for available and default values.
 #'   \item mdl_h1_control: List with unrestricted model options. See \code{\link{HMmdl}}, \code{\link{MSARmdl}}, or \code{\link{MSVARmdl}} documentation for available and default values.
@@ -110,20 +134,20 @@ estimMdl <- function(Y, p, q, k, control = list()){
 #'
 #' @return List of class \code{LMCLRTest} (\code{S3} object) with attributes including: 
 #' \itemize{
-#'   \item mdl_h0: List with restricted model attributes. See \code{\link{Nmdl}}, \code{\link{ARmdl}}, \code{\link{VARmdl}}, \code{\link{HMmdl}}, \code{\link{MSARmdl}}, or \code{\link{MSVARmdl}} documentation for return values.
-#'   \item mdl_h0: List with unrestricted model attributes. See \code{\link{HMmdl}}, \code{\link{MSARmdl}}, or \code{\link{MSVARmdl}} documentation for return values.
+#'   \item mdl_h0: List with restricted model attributes. 
+#'   \item mdl_h1: List with unrestricted model attributes. 
 #'   \item LRT_0: Value of test statistic from observed data.
 #'   \item LRN: A (\code{N x 1}) vector of test statistics from data simulated under the null hypothesis.
 #'   \item pval: P-value of Local Monte Carlo Likelihood Ratio Test.
-#'   \item LRN_cv: Vector with 90\%, 95\%, and 99\% Monte Carlo critical values (from vector \code{LRN}).
+#'   \item LRN_cv: Vector with 90\%, 95\%, and 99\% Monte Carlo simulated critical values (from vector \code{LRN}). These are not asymptotic critical values. 
 #'   \item control: List with test procedure options used.
 #' }
 #'
 #' @references Rodriguez-Rondon, Gabriel and Jean-Marie Dufour. 2022. "Simulation-Based Inference for Markov Switching Models” \emph{JSM Proceedings, Business and Economic Statistics Section: American Statistical Association}.
-#' @references Rodriguez-Rondon, Gabriel and Jean-Marie Dufour. 2023. “Monte Carlo Likelihood Ratio Tests for Markov Switching Models.” \emph{Unpublished manuscript}.
+#' @references Rodriguez-Rondon, Gabriel and Jean-Marie Dufour. 2024. “Monte Carlo Likelihood Ratio Tests for Markov Switching Models.” \emph{Unpublished manuscript}.
 #' @example /inst/examples/LMCLRTest_examples.R
 #' @export
-LMCLRTest <- function(Y, p, k0, k1, control = list()){
+LMCLRTest <- function(Y, p, k0, k1, Z = NULL, control = list()){
   # ----- Set control values
   con <- list(N = 99,
               burnin = 100,
@@ -145,8 +169,8 @@ LMCLRTest <- function(Y, p, k0, k1, control = list()){
     stop("Observations Y must be a (T x q) matrix.") 
   }
   # ----- Estimate models using observed data
-  mdl_h0 <- estimMdl(Y, p, q, k0, con$mdl_h0_control)
-  mdl_h1 <- estimMdl(Y, p, q, k1, con$mdl_h1_control)
+  mdl_h0 <- estimMdl(Y, p, q, k0, Z, con$mdl_h0_control)
+  mdl_h1 <- estimMdl(Y, p, q, k1, Z, con$mdl_h1_control)
   con$mdl_h0_control <- mdl_h0$control
   con$mdl_h1_control <- mdl_h1$control
   # ----- Optional model convergence checks
@@ -176,6 +200,11 @@ LMCLRTest <- function(Y, p, k0, k1, control = list()){
   }
   names(LRT_0) <- c("LRT_0")
   # ----- Simulate sample null distribution
+  if (is.null(Z)==FALSE){
+    Zsim <- Z[(p+1):nrow(Z),,drop=F]
+  }else{
+    Zsim <- Z
+  }
   mdl_h0_null_cont <- con$mdl_h0_control
   mdl_h1_null_cont <- con$mdl_h1_control
   if (is.null(con$use_diff_init_sim)==FALSE){
@@ -183,9 +212,9 @@ LMCLRTest <- function(Y, p, k0, k1, control = list()){
     mdl_h1_null_cont$use_diff_init <- con$use_diff_init_sim
   }
   if (con$workers>0){
-    LRN <- LR_samp_dist_par(mdl_h0, k1, con$N, con$burnin, mdl_h0_null_cont, mdl_h1_null_cont, con$workers)
+    LRN <- LR_samp_dist_par(mdl_h0, k1, con$N, con$burnin, Zsim, mdl_h0_null_cont, mdl_h1_null_cont, con$workers)
   }else{
-    LRN <- LR_samp_dist(mdl_h0, k1, con$N, con$burnin, mdl_h0_null_cont, mdl_h1_null_cont) 
+    LRN <- LR_samp_dist(mdl_h0, k1, con$N, con$burnin, Zsim, mdl_h0_null_cont, mdl_h1_null_cont) 
   }
   # ----- get critical values
   LRN     <- as.matrix(sort(LRN))
@@ -213,12 +242,12 @@ LMCLRTest <- function(Y, p, k0, k1, control = list()){
 #' @keywords internal
 #' 
 #' @references Rodriguez-Rondon, Gabriel and Jean-Marie Dufour. 2022. "Simulation-Based Inference for Markov Switching Models” \emph{JSM Proceedings, Business and Economic Statistics Section: American Statistical Association}.
-#' @references Rodriguez-Rondon, Gabriel and Jean-Marie Dufour. 2023. “Monte Carlo Likelihood Ratio Tests for Markov Switching Models.” \emph{Unpublished manuscript}.
+#' @references Rodriguez-Rondon, Gabriel and Jean-Marie Dufour. 2024. “Monte Carlo Likelihood Ratio Tests for Markov Switching Models.” \emph{Unpublished manuscript}.
 #' 
 #' @export
 MMC_bounds <- function(mdl_h0, con){
-  theta_0 <- mdl_h0$theta
-  k0 <- mdl_h0$k
+  theta_0   <- mdl_h0$theta
+  k0        <- mdl_h0$k
   # ----- Define lower & upper bounds for search
   theta_low = theta_0 - con$eps
   theta_upp = theta_0 + con$eps
@@ -260,12 +289,13 @@ MMC_bounds <- function(mdl_h0, con){
 #' @title Maximized Monte Carlo Likelihood Ratio Test
 #'
 #' @description This function performs the Maximized Monte Carlo likelihood ratio 
-#' test (MMC-LRT) proposed in Rodriguez-Rondon & Dufour (2022).
+#' test (MMC-LRT) proposed in Rodriguez-Rondon & Dufour (2024).
 #' 
-#' @param Y  Series to be tested. Must be a (\code{T x q}) matrix.
+#' @param Y  Series to be tested. Must be a (\code{T x q}) matrix  where T is the number of time observations and q is the number of variables.
 #' @param p  Number of autoregressive lags. Must be greater than or equal to 0. 
 #' @param k0 Number of regimes under null hypothesis. Must be greater than or equal to 1.
 #' @param k1 Number of regimes under alternative hypothesis. Must be greater than \code{k0}.
+#' @param Z  Exogenous regressors. Optional input and default is NULL. When used, it should be a (\code{T x qz}) matrix where T is the number of time observations and q is the number of exogenous variables.
 #' @param control List with test procedure options including: 
 #' \itemize{
 #'   \item N: Integer determining the number of Monte Carlo simulations. Default is set to \code{99} as in paper.
@@ -287,25 +317,25 @@ MMC_bounds <- function(mdl_h0, con){
 #'   \item mdl_h0_control: List with restricted model options. See \code{\link{Nmdl}}, \code{\link{ARmdl}}, \code{\link{VARmdl}}, \code{\link{HMmdl}}, \code{\link{MSARmdl}}, or \code{\link{MSVARmdl}} documentation for available and default values.
 #'   \item mdl_h1_control: List with unrestricted model options. See \code{\link{HMmdl}}, \code{\link{MSARmdl}}, or \code{\link{MSVARmdl}} documentation for available and default values.
 #'   \item use_diff_init_sim: Value which determines the number of initial values to use when estimating models for null distribution. Default is set to use the same as specified in \code{mdl_h0_control} and \code{mdl_h1_control}.
-#'   \item type_control: List with optimization algorithm options. See \code{\link[pso]{psoptim}}, \code{\link[GenSA]{GenSA}}, \code{\link[GA]{ga}}. Default is to set \code{list(maxit = 200)} so that maximum number of iterations is \code{200}.
+#'   \item optim_control: List with optimization algorithm options. See \code{\link[pso]{psoptim}}, \code{\link[GenSA]{GenSA}}, \code{\link[GA]{ga}}. Default is to set \code{list(maxit = 200)} so that maximum number of iterations is \code{200}.
 #' }
 #'
 #' @return List of class \code{LMCLRTest} (\code{S3} object) with attributes including: 
 #' \itemize{
-#'   \item mdl_h0: List with restricted model attributes. See \code{\link{Nmdl}}, \code{\link{ARmdl}}, \code{\link{VARmdl}}, \code{\link{HMmdl}}, \code{\link{MSARmdl}}, or \code{\link{MSVARmdl}} documentation for return values.
-#'   \item mdl_h0: List with unrestricted model attributes. See \code{\link{HMmdl}}, \code{\link{MSARmdl}}, or \code{\link{MSVARmdl}} documentation for return values.
+#'   \item mdl_h0: List with restricted model attributes. 
+#'   \item mdl_h1: List with unrestricted model attributes. 
 #'   \item LRT_0: Value of test statistic from observed data.
 #'   \item LRN: A (\code{N x 1}) vector of test statistics from data simulated under the null hypothesis.
 #'   \item pval: P-value of Local Monte Carlo Likelihood Ratio Test.
-#'   \item LRN_cv: Vector with 90\%, 95\%, and 99\% Monte Carlo critical values (from vector \code{LRN}).
+#'   \item LRN_cv: Vector with 90\%, 95\%, and 99\% Monte Carlo simulated critical values (from vector \code{LRN}). These are not asymptotic critical values. 
 #'   \item control: List with test procedure options used.
 #' }
 #'
 #' @references Rodriguez-Rondon, Gabriel and Jean-Marie Dufour. 2022. "Simulation-Based Inference for Markov Switching Models” \emph{JSM Proceedings, Business and Economic Statistics Section: American Statistical Association}.
-#' @references Rodriguez-Rondon, Gabriel and Jean-Marie Dufour. 2023. “Monte Carlo Likelihood Ratio Tests for Markov Switching Models.” \emph{Unpublished manuscript}.
+#' @references Rodriguez-Rondon, Gabriel and Jean-Marie Dufour. 2024. “Monte Carlo Likelihood Ratio Tests for Markov Switching Models.” \emph{Unpublished manuscript}.
 #' @example /inst/examples/MMCLRTest_examples.R
 #' @export
-MMCLRTest <- function(Y, p, k0, k1, control = list()){
+MMCLRTest <- function(Y, p, k0, k1, Z = NULL, control = list()){
   # ----- Set control values
   con <- list(N = 99,
               burnin = 100,
@@ -327,7 +357,7 @@ MMCLRTest <- function(Y, p, k0, k1, control = list()){
               mdl_h1_control = list(getSE = TRUE),
               use_diff_init_sim = NULL,
               maxit = 50,
-              type_control = list())
+              optim_control = list())
   # ----- Perform some checks for controls
   nmsC <- names(con)
   con[(namc <- names(control))] <- control
@@ -346,8 +376,8 @@ MMCLRTest <- function(Y, p, k0, k1, control = list()){
     warning("getSE was changed to be 'TRUE' because CI_union is 'TRUE'.")
   }
   # ----- Estimate models using observed data
-  mdl_h0 <- estimMdl(Y, p, q, k0, con$mdl_h0_control)
-  mdl_h1 <- estimMdl(Y, p, q, k1, con$mdl_h1_control)
+  mdl_h0 <- estimMdl(Y, p, q, k0, Z, con$mdl_h0_control)
+  mdl_h1 <- estimMdl(Y, p, q, k1, Z, con$mdl_h1_control)
   con$mdl_h0_control <- mdl_h0$control
   con$mdl_h1_control <- mdl_h1$control
   # ----- Optional model convergence checks
@@ -374,33 +404,41 @@ MMCLRTest <- function(Y, p, k0, k1, control = list()){
     mdl_h0_null_cont$use_diff_init <- con$use_diff_init_sim
     mdl_h1_null_cont$use_diff_init <- con$use_diff_init_sim
   }
+  if (is.null(Z)==FALSE){
+    Zsim <- Z[(p+1):nrow(Z),,drop=F]
+    exog <- TRUE
+  }else{
+    Zsim <- Z
+    exog <- FALSE
+  }
   if (con$type=="pso"){
     # Set PSO specific controls
-    con$type_control$trace.stats <- TRUE
-    con$type_control$trace <- as.numeric(con$silence==FALSE)
-    con$type_control$abstol <- -con$threshold_stop
-    con$type_control$maxf <- con$maxit
+    con$optim_control$trace.stats <- TRUE
+    con$optim_control$trace <- as.numeric(con$silence==FALSE)
+    con$optim_control$abstol <- -con$threshold_stop
+    con$optim_control$maxf <- con$maxit
+    con$optim_control$REPORT <- 1
     # begin optimization
     mmc_out   <- pso::psoptim(par = theta_0, fn = MMCLRpval_fun_min, lower = theta_low, upper = theta_upp, 
-                              gr = NULL, control = con$type_control,
+                              gr = NULL, control = con$optim_control,
                               mdl_h0 = mdl_h0, k1 = k1, LT_h1 = mdl_h1$logLike, N = con$N, burnin = con$burnin, workers = con$workers,
                               lambda = con$lambda, stationary_constraint = con$stationary_constraint, 
-                              thtol = mdl_h1$control$thtol, mdl_h0_control = mdl_h0_null_cont, 
+                              thtol = mdl_h1$control$thtol, Z = Zsim, exog = exog, mdl_h0_control = mdl_h0_null_cont, 
                               mdl_h1_control = mdl_h1_null_cont)
     theta     <- mmc_out$par
     pval      <- -mmc_out$value
   }else if(con$type=="GenSA"){
     # Set GenSA specific controls
-    con$type_control$trace.mat <- TRUE
-    con$type_control$verbose <- con$silence==FALSE
-    con$type_control$threshold.stop <- -con$threshold_stop
-    con$type_control$max.call <- con$maxit
+    con$optim_control$trace.mat <- TRUE
+    con$optim_control$verbose <- con$silence==FALSE
+    con$optim_control$threshold.stop <- -con$threshold_stop
+    con$optim_control$max.call <- con$maxit
     # begin optimization
     mmc_out   <- GenSA::GenSA(par = theta_0, fn = MMCLRpval_fun_min, lower = theta_low, upper = theta_upp, 
-                              control = con$type_control,
+                              control = con$optim_control,
                               mdl_h0 = mdl_h0, k1 = k1, LT_h1 = mdl_h1$logLike, N = con$N, burnin = con$burnin, workers = con$workers,
                               lambda = con$lambda, stationary_constraint = con$stationary_constraint, 
-                              thtol = mdl_h1$control$thtol, mdl_h0_control = mdl_h0_null_cont, 
+                              thtol = mdl_h1$control$thtol, Z = Zsim, exog = exog, mdl_h0_control = mdl_h0_null_cont, 
                               mdl_h1_control = mdl_h1_null_cont)
     theta     <- mmc_out$par
     pval      <- -mmc_out$value
@@ -409,7 +447,7 @@ MMCLRTest <- function(Y, p, k0, k1, control = list()){
     mmc_out   <- GA::ga(type = "real-valued", fitness = MMCLRpval_fun, 
                       mdl_h0 = mdl_h0, k1 = k1, LT_h1 = mdl_h1$logLike, N = con$N, burnin = con$burnin, workers = con$workers,
                       lambda = con$lambda, stationary_constraint = con$stationary_constraint, 
-                      thtol = mdl_h1$control$thtol, mdl_h0_control = mdl_h0_null_cont, 
+                      thtol = mdl_h1$control$thtol, Z = Zsim,exog = exog,  mdl_h0_control = mdl_h0_null_cont, 
                       mdl_h1_control = mdl_h1_null_cont,
                       lower = theta_low, upper = theta_upp, 
                       maxiter = con$maxit, maxFitness = con$threshold_stop, 
@@ -417,14 +455,40 @@ MMCLRTest <- function(Y, p, k0, k1, control = list()){
     theta     <- as.matrix(mmc_out@solution[1,])
     pval      <- mmc_out@fitnessValue
   }else if(con$type=="gridSearch"){
-    # Grid Search: not ready
+    stop("Optim method 'gridSearch' is not available yet. Please use 'pso', 'GenSA', or 'GA' for 'type' in control List. ")
+    # LT_h1 <- mdl_h1$logLike
+    # LRT_0s <- matrix(0,con$maxit,1)
+    # mmc_params_h0 <- matrix(0,con$maxit,length(theta_0))
+    # for (xp in 1:length(theta_0)){
+    #   mmc_params_h0[,xp] <- runif(con$maxit,min = theta_low[xp], max = theta_upp[xp])  
+    # }
+    # # Need to write soemthing that will make sure process is stationary, P has columns that sum to 1
+    # mmc_pval_mat <- matrix(0,con$maxit,1)
+    # LRN_ls <- list()
+    # for (xs in 1:nrow(mmc_params_h0)){
+    #    mdl_h0_tmp <- mdledit(mdl_h0,mmc_params_h0[xs,],p,q,k0,exog)
+    #    LRT_0s[xs,]  <- compu_tstat(mmc_params_h0[xs,], mdl_h0_tmp, LT_h1, p, q, k0, exog)
+    #    if (con$workers>0){
+    #      LRN <- LR_samp_dist_par(mdl_h0_tmp, k1, con$N, con$burnin, Zsim, mdl_h0_null_cont, mdl_h1_null_cont, con$workers)
+    #    }else{
+    #      LRN <- LR_samp_dist(mdl_h0, k1, con$N, con$burnin, Zsim, mdl_h0_null_cont, mdl_h1_null_cont) 
+    #    }
+    #    LRN_ls[[xs]] <- LRN
+    #    mmc_pval_mat[xs,] <- MCpval(LRT_0s[xs,],LRN)
+    #    if (mmc_pval_mat[xs,]>con$threshold_stop){
+    #      break
+    #    }
+    # }
+    # pval <- mmc_pval_mat[which.max(mmc_pval_mat)[1],]
+    # theta <- mmc_params_h0[which.max(mmc_pval_mat)[1],]
+    # LRT_0 <- LRT_0s[which.max(mmc_pval_mat)[1],]
   }
   # ----- get test output using optimization output params
   theta_h0 <- theta
   theta_h1 <- mdl_h1$theta
   names(theta_h0) <- names(mdl_h0$theta)
   names(theta_h1) <- names(mdl_h1$theta)
-  mdl_h0_mmc <- mdledit(mdl_h0, theta_h0, p, q, k0)
+  mdl_h0_mmc <- mdledit(mdl_h0, theta_h0, p, q, k0, exog)
   mdl_h0_mmc$logLike <- logLik(mdl_h0_mmc)
   mdl_h0_mmc$AIC <- stats::AIC(mdl_h0_mmc)
   mdl_h0_mmc$BIC <- stats::BIC(mdl_h0_mmc)
@@ -432,7 +496,7 @@ MMCLRTest <- function(Y, p, k0, k1, control = list()){
     mdl_h0_mmc <- thetaSE(mdl_h0_mmc)
   }
   # Compute test stats
-  LRT_0 = compu_tstat(theta_h0, mdl_h0, mdl_h1$logLike, p, q, k0)
+  LRT_0 = compu_tstat(theta_h0, mdl_h0, mdl_h1$logLike, p, q, k0, exog)
   names(LRT_0) <- c("LRT_0")
   # ----- organize test output
   MMCLRTest_output <- list(mdl_h0 = mdl_h0, mdl_h1 = mdl_h1, mdl_h0_mmc = mdl_h0_mmc, mdl_h1_mmc = mdl_h1, 
@@ -445,116 +509,3 @@ MMCLRTest <- function(Y, p, k0, k1, control = list()){
 
 
 
-#' @title Bootstrap Likelihood Ratio Test
-#' 
-#' @description This function performs the bootstrap likelihood ratio 
-#' test discussed in Qu & Zhuo (2021) and Kasahara & Shimotsu (2018).
-#' 
-#' @param Y  Series to be tested. Must be a (\code{T x q}) matrix.
-#' @param p  Number of autoregressive lags. Must be greater than or equal to 0. 
-#' @param k0 Number of regimes under null hypothesis. Must be greater than or equal to 1.
-#' @param k1 Number of regimes under alternative hypothesis. Must be greater than \code{k0}.
-#' @param control List with test procedure options including: 
-#' \itemize{
-#'   \item B: Integer determining the number of bootstrap simulations. Default is set to \code{999}.
-#'   \item burnin: Number of simulated observations to remove from beginning. Default is \code{100}.
-#'   \item converge_check: String of NULL determining if convergence of model(s) should be verified. Allowed inputs are: "null", "alt", "both", or \code{NULL}. If \code{NULL} (default) no model convergence is verified.
-#'   \item workers: Integer determining the number of workers to use for parallel computing version of test. Note that parallel pool must already be open. Default is \code{0}.
-#'   \item mdl_h0_control: List with restricted model options. See \code{\link{Nmdl}}, \code{\link{ARmdl}}, \code{\link{VARmdl}}, \code{\link{HMmdl}}, \code{\link{MSARmdl}}, or \code{\link{MSVARmdl}} documentation for available and default values.
-#'   \item mdl_h1_control: List with unrestricted model options. See \code{\link{HMmdl}}, \code{\link{MSARmdl}}, or \code{\link{MSVARmdl}} documentation for available and default values.
-#' }
-#'
-#' @return List of class \code{LMCLRTest} (\code{S3} object) with attributes including: 
-#' \itemize{
-#'   \item mdl_h0: List with restricted model attributes. See \code{\link{Nmdl}}, \code{\link{ARmdl}}, \code{\link{VARmdl}}, \code{\link{HMmdl}}, \code{\link{MSARmdl}}, or \code{\link{MSVARmdl}} documentation for return values.
-#'   \item mdl_h0: List with unrestricted model attributes. See \code{\link{HMmdl}}, \code{\link{MSARmdl}}, or \code{\link{MSVARmdl}} documentation for return values.
-#'   \item LRT_0: Value of test statistic from observed data.
-#'   \item LRN: A (\code{N x 1}) vector of test statistics from data simulated under the null hypothesis.
-#'   \item pval: P-value of Local Monte Carlo Likelihood Ratio Test.
-#'   \item LRN_cv: Vector with 90\%, 95\%, and 99\% Monte Carlo critical values (from vector \code{LRN}).
-#'   \item control: List with test procedure options used.
-#' }
-#' 
-#' @references Qu, Zhongjun, and Fan Zhuo. 2021. “Likelihood Ratio-Based Tests for Markov Regime Switching.” \emph{The Review of Economic Studies} 88 (2): 937–968.
-#' @references Kasahara, Hiroyuk, and Katsum Shimotsu. 2018. “Testing the number of regimes in Markov regime switching models.” \emph{arXiv preprint arXiv:1801.06862}.
-#' @references Rodriguez-Rondon, Gabriel and Jean-Marie Dufour. 2023. “Monte Carlo Likelihood Ratio Tests for Markov Switching Models.” \emph{Unpublished manuscript}.
-#' 
-#' @export
-BootLRTest <- function(Y, p, k0, k1, control = list()){
-  # ----- Set control values
-  con <- list(B = 999,
-              burnin = 100,
-              converge_check = NULL,
-              workers = 0,
-              mdl_h0_control = list(),
-              mdl_h1_control = list())
-  # *** TO DO: Add default values to reproduce Qu & Zhou (2021) Parametric bootstrap (i.e. use method 'mle' and parameter restrictions on P.)
-  # ----- Perform some checks for controls
-  nmsC <- names(con)
-  con[(namc <- names(control))] <- control
-  if (length(noNms <- namc[!namc %in% nmsC])){
-    warning("unknown names in control: ", paste(noNms,collapse=", ")) 
-  }
-  # ----- Perform other checks
-  if (is.matrix(Y)){
-    q <- ncol(Y)
-  }else{
-    stop("Observations Y must be a (T x 1) matrix.") 
-  }
-  if(q>1){
-    stop("BootLRTest() is only designed for univariate models (i.e. q=1). Consider using LMCLRTest() or MMCLRTest() procedures.")
-  }
-  if ((k1-k0)>1){
-    stop("BootLRTest() is only designed to test M vs. M+1 regimes. Consider using LMCLRTest() or MMCLRTest() procedures.")
-  }
-  # ----- Estimate models using observed data
-  mdl_h0 <- estimMdl(Y, p, q, k0, con$mdl_h0_control)
-  mdl_h1 <- estimMdl(Y, p, q, k1, con$mdl_h1_control)
-  con$mdl_h0_control <- mdl_h0$control
-  con$mdl_h1_control <- mdl_h1$control
-  # ----- Optional model convergence checks
-  if (is.null(con$converge_check)==FALSE){
-    if ((con$converge_check=="null") & (mdl_h0$converged==FALSE)){
-      stop("Model under null hypothesis did not converge. Run again to use different initial values and/or increase 'maxit' for restricted model.")
-    }
-    if ((con$converge_check=="alt") & (mdl_h1$converged==FALSE)){
-      stop("Model under alternative hypothesis did not converge. Run again to use different initial values and/or increase 'maxit' for unrestricted model.")
-    }
-    if ((con$converge_check=="both") & ((mdl_h0$converged==FALSE) | (mdl_h1$converged==FALSE))){
-      stop("Model did not converge. Run again to use different initial values and/or increase 'maxit' for each models.")
-    }
-  }
-  # ----- Compute test statistic (LRT_0)
-  logL0 <- mdl_h0$logLike
-  logL1 <- mdl_h1$logLike
-  theta_h0 <- mdl_h0$theta
-  theta_h1 <- mdl_h1$theta
-  LRT_0 <- -2*(logL0-logL1)
-  # ----- Perform check of test stat and model parameters
-  if ((is.finite(LRT_0)==FALSE) | (any(is.finite(theta_h0)==FALSE)) | (any(is.finite(theta_h1)==FALSE))){
-    stop("LRT_0 or model parameters are not finite. Run again to use different initial values") 
-  }
-  if (LRT_0<0){
-    stop("LRT_0 is negative. Run again to use different initial values")
-  }
-  names(LRT_0) <- c("LRT_0")
-  # ----- Simulate sample null distribution
-  if (con$workers>0){
-    LRN <- LR_samp_dist_par(mdl_h0, k1, con$B, con$burnin, con$mdl_h0_control, con$mdl_h1_control, con$workers)
-  }else{
-    LRN <- LR_samp_dist(mdl_h0, k1, con$B, con$burnin, con$mdl_h0_control, con$mdl_h1_control) 
-  }
-  # ----- Compute p-value
-  pval <- sum(LRN>LRT_0)/con$B # [eq. 4.62] (Davidson & MacKinnon, 2004)
-  # ----- get critical values
-  LRN     <- as.matrix(sort(LRN))
-  LRN_cv  <- LRN[round(c(0.90,0.95,0.99)*nrow(LRN)),]
-  names(LRN_cv)  <- paste0(c("0.90","0.95","0.99"), "%")
-  # ----- Organize output
-  BootLRTest_output <- list(mdl_h0 = mdl_h0, mdl_h1 = mdl_h1, LRT_0 = LRT_0, LRN = LRN,
-                            pval = pval, LRN_cv = LRN_cv, control = con)
-  class(BootLRTest_output) <- "BootLRTest"
-  return(BootLRTest_output)
-}
-
-
